
#include <windows.h>
#include <stdio.h>
#include "stream.h"

#if defined (WIN32)
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif

#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32

HINSTANCE hInst;   // current instance

LPCTSTR lpszAppName = "MyApp";
LPCTSTR lpszTitle   = "MIDI Stream"; 


BOOL RegisterWin95( CONST WNDCLASS* lpwc );


int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG      msg;
   HWND     hWnd; 
   WNDCLASS wc;

   wc.style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
   wc.lpfnWndProc   = (WNDPROC)WndProc;       
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;              
   wc.hIcon         = LoadIcon (hInstance, lpszAppName); 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName;              

   if ( IS_WIN95 )
   {
      if ( !RegisterWin95( &wc ) )
         return( FALSE );
   }
   else if ( !RegisterClass( &wc ) )
      return( FALSE );

   hInst = hInstance; 

   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW | WS_VSCROLL, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 
}


BOOL RegisterWin95( CONST WNDCLASS* lpwc )
{
	WNDCLASSEX wcex;

   wcex.style         = lpwc->style;
   wcex.lpfnWndProc   = lpwc->lpfnWndProc;
   wcex.cbClsExtra    = lpwc->cbClsExtra;
   wcex.cbWndExtra    = lpwc->cbWndExtra;
   wcex.hInstance     = lpwc->hInstance;
   wcex.hIcon         = lpwc->hIcon;
   wcex.hCursor       = lpwc->hCursor;
   wcex.hbrBackground = lpwc->hbrBackground;
   wcex.lpszMenuName  = lpwc->lpszMenuName;
   wcex.lpszClassName = lpwc->lpszClassName;

   // Added elements for Windows 95.
   //...............................
   wcex.cbSize = sizeof(WNDCLASSEX);
   wcex.hIconSm = LoadImage(wcex.hInstance, lpwc->lpszClassName, 
                            IMAGE_ICON, 16, 16,
                            LR_DEFAULTCOLOR );
			
   return RegisterClassEx( &wcex );
}


#define USR_SHORTMSG  (WM_USER+1)

#define MSG_LEN          1024
#define DATABLOCK_SIZE   1024L

char       msg[MSG_LEN+1];

HWND       hListBox = NULL;
MMRESULT   rc;

UINT       nDevId = 1;
HMIDISTRM  hms    = NULL;
MIDIHDR*   pmh    = NULL;

typedef struct
{
   DWORD ms;        // time in milliseconds
   DWORD dwMidiMsg; // short MIDI message
} MIDIMSG;

#define SONG_LEN  6 

MIDIEVENT MidiEvent[SONG_LEN] =
   {  0, 0, MEVT_F_SHORT | MEVT_F_CALLBACK | 0x403c90, 0,  // C on  (C3)
     10, 0, MEVT_F_SHORT | MEVT_F_CALLBACK | 0x003c90, 0,  // C off
     10, 0, MEVT_F_SHORT | MEVT_F_CALLBACK | 0x404c90, 0,  // G on
     10, 0, MEVT_F_SHORT | MEVT_F_CALLBACK | 0x004c90, 0,  // G off
     10, 0, MEVT_F_SHORT | MEVT_F_CALLBACK | 0x404090, 0,  // E on
     10, 0, MEVT_F_SHORT | MEVT_F_CALLBACK | 0x004090, 0   // E off
   };


/*****************************************************************************
*  WndProc
*
*
*
******************************************************************************/

LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
   switch( uMsg )
   {
      case WM_CREATE :
              // allocate MIDIHDR buffer block
	           //..............................

              pmh = HeapAlloc( GetProcessHeap(),                         
	                            HEAP_ZERO_MEMORY,                         
	                            sizeof(MIDIHDR) );                        
              if (pmh)                                                  
              {                                                            
                  pmh->lpData = HeapAlloc( GetProcessHeap(),             
                                           HEAP_ZERO_MEMORY,             
                                           DATABLOCK_SIZE);                       
                  pmh->dwBufferLength = DATABLOCK_SIZE;
              }

              // Create ListBox
              //...............

              hListBox = CreateWindow( "LISTBOX", "",    
                                       WS_CHILD | LBS_NOTIFY | 
                                       WS_VSCROLL | WS_BORDER | 
                                       WS_VISIBLE | LBS_NOINTEGRALHEIGHT, 
                                       0, 0, 
                                       0, 0,  
                                       hWnd,              
                                       (HMENU)101,              
                                       hInst,         
                                       NULL );
              break;

      case WM_SIZE :
              MoveWindow( hListBox, 0, 0, 
                          LOWORD( lParam ), 
                          HIWORD( lParam ), TRUE );
              break;

      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                 case IDM_TEST:
                        {
                           MIDIPROPTEMPO   mpt;
                           MIDIPROPTIMEDIV mptd;
                           
                           // open MIDI stream
                           //.................
                           
                           rc = midiStreamOpen(&hms, &nDevId, 1,    
                                               (DWORD)hWnd, (DWORD)NULL, CALLBACK_WINDOW);

                           if (rc != MMSYSERR_NOERROR)
                           {
                               midiOutGetErrorText(rc, msg, MSG_LEN);
                               MessageBox(hWnd, msg, NULL, MB_OK);
                               break;
                           }

                           // pause playback of MIDI stream
                           //..............................

                           midiStreamPause(hms);

                           // display default stream properties
                           //..................................

                           mpt.cbStruct  = sizeof(MIDIPROPTEMPO);
                           mptd.cbStruct = sizeof(MIDIPROPTIMEDIV);

                           midiStreamProperty(hms, (LPBYTE)&mpt,  MIDIPROP_GET | MIDIPROP_TEMPO);
                           midiStreamProperty(hms, (LPBYTE)&mptd, MIDIPROP_GET | MIDIPROP_TIMEDIV);
                           
                           sprintf(msg, "Tempo:%ld  Time Div:%ld",
                                   mpt.dwTempo, mptd.dwTimeDiv);
                           SendMessage(hListBox, LB_ADDSTRING, 0, (LPARAM)msg); 

                           // Load data into MIDIHDR
                           //.......................

                           pmh->dwBytesRecorded = SONG_LEN * sizeof(MIDIEVENT);
                           memcpy(pmh->lpData, MidiEvent, pmh->dwBytesRecorded);

                           // send MIDI stream to MIDI output device and
                           // restart playback 
                           //...........................................

                           rc = midiOutPrepareHeader(hms, pmh, sizeof(MIDIHDR));

                           if (rc == MMSYSERR_NOERROR)
                               rc = midiStreamRestart(hms);

                           if (rc == MMSYSERR_NOERROR)
                               rc = midiStreamOut(hms, pmh, sizeof(MIDIHDR));
                           
                           if (rc != MMSYSERR_NOERROR)
                           {
                               midiOutGetErrorText(rc, msg, MSG_LEN);
                               MessageBox(hWnd, msg, NULL, MB_OK);
                               break;
                           }

                           // call midiStreamPosition() and display current position
                        }
                        break;

                 case IDM_ABOUT :
                        DialogBox( hInst, "AboutBox", hWnd, About );
                        break;

                 case IDM_EXIT :
                        DestroyWindow( hWnd );
                        break;
              }
              break;

      case MM_STREAM_DONE:
      case MM_MOM_DONE:
              {
                 // unprepeare the data buffer block
                 // and close the MIDI output device
                 //.................................

                 midiOutUnprepareHeader(hms, pmh, sizeof(MIDIHDR));
                 midiStreamStop(hms);
                 midiStreamClose(hms);
              }
              break;

      case WM_DESTROY :
              // unprepeare the data buffer block
              // and close the MIDI output device
              //.................................

              midiOutUnprepareHeader(hms, pmh, sizeof(MIDIHDR));
              midiStreamStop(hms);
              midiStreamClose(hms);
              
              // free the MIDIHDR buffer block
	           //..............................

              if (pmh != NULL)
              {
                  HeapFree(GetProcessHeap(), 0, pmh->lpData);                 
                  HeapFree(GetProcessHeap(), 0, pmh);
                  pmh = NULL;
	           }

              PostQuitMessage(0);
              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );               
}



LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}









